unit regexpo;

interface

uses
  Windows, Registry, Classes, SysUtils;

{$I-}
{$LONGSTRINGS ON}

{
  Regexpo

  Author: Arthur Hoornweg (arthur.hoornweg@email.de)
  Version: 1.00, May 1998
  Modificated by: olesio (oles@o2.pl), 8 March 2009
  O/S: Windows 95, 98, ME, NT, 2000, XP
       Delphi 2+
  Function:
    This unit allows you to backup a branch of the registry into a *.REG file,
    that is compatible with "regedit".
    Double-clicking such a file in the explorer will import it.


  Example:
    ExportRegistryBranch(HKEY_LOCAL_MACHINE,'SOFTWARE\Borland\Delphi','A:\DELPHI.REG')

  (c) 1998 A.M. Hoornweg. All rights reserved.

  You may use this software for all purposes, both commercial and noncommercial,
  as long as proper credit is given. The sourcecode may be distributed freely,
  as long as this copyright is included and no more than a marginal fee is asked.


  Disclaimer:

  I accept no responsibility whatsoever for any damages caused by these routines.
  Use them at your own risk. If you find any bugs, please let me know.
}


procedure ExportRegistryBranch(RootSection: HKEY; Regroot: string;
  FileName: string);
procedure ExportRegistryBranchToStringList(RootSection: HKEY; Regroot: string;
  SL: TStringList);

implementation

function DblBackSlash(t: string): string;
var
  k: longint;
begin
  Result := t;
  for k := Length(t) downto 1 do
    if Result[k] = '\' then Insert('\', Result, k);
end;

function AddBackSlashBeforeQuote(t: string): string;
var
  k: longint;
begin
  Result := t;
  for k := Length(t) downto 1 do
    if Result[k] = '"' then Insert('\', Result, k);
end;

procedure ExportRegistryBranch(RootSection: HKEY; Regroot: string;
  FileName: string);
var
  reg: TRegistry;
  f: Textfile;
  p: PChar;

  procedure ProcessBranch(root: string);
  var
    values, keys: TStringList;
    i, j, k: longint;
    s, t: string;
  begin
    Writeln(f);
    case rootsection of
      HKEY_CLASSES_ROOT: s   := 'HKEY_CLASSES_ROOT';
      HKEY_CURRENT_USER: s   := 'HKEY_CURRENT_USER';
      HKEY_LOCAL_MACHINE: s  := 'HKEY_LOCAL_MACHINE';
      HKEY_USERS: s          := 'HKEY_USERS';
      HKEY_PERFORMANCE_DATA: s := 'HKEY_PERFORMANCE_DATA';
      HKEY_CURRENT_CONFIG: s := 'HKEY_CURRENT_CONFIG';
      HKEY_DYN_DATA: s       := 'HKEY_DYN_DATA';
    end;
    Writeln(f, '[' + s + '\' + root + ']');

    reg.OpenKey(root, False);
    try
      values := TStringList.Create;
      try
        keys := TStringList.Create;
        try
          reg.GetValuenames(values);
          reg.GetKeynames(keys);

          for i := 0 to values.Count - 1 do
          begin
            s := values[i];
            t := s;
            if s = '' then s := '@' + '='
            else
            s := '"' + AddBackSlashBeforeQuote(Dblbackslash(s)) + '"' + '=';
            Write(f, s);

            case reg.Getdatatype(t) of

              rdString:
                Writeln(f, '"' + AddBackSlashBeforeQuote(Dblbackslash(reg.ReadString(t))) + '"');

              rdExpandString:
                begin
                  s := Reg.ReadString(t);
                  j := Reg.GetDataSize(t);
                  Write(f, 'hex(2):');
                  for k := 1 to j do
                  begin
                    Write(f, LowerCase(IntToHex(Ord(s[k]), 2)));
                    if k <> j then
                    begin
                      Write(f, ',');
                      if (k > 0) and ((k mod 23) = 0) then
                        begin
                        Writeln(f, '\');
                        Write(f, '  ');
                        end;
                    end;
                  end;
                  Writeln(f);
                end;

              rdInteger:
                Writeln(f, 'dword:' + LowerCase(IntToHex(reg.readinteger(t), 8)));

              rdBinary:
                begin
                  Write(f, 'hex:');
                  j := reg.GetDataSize(t);
                  GetMem(p, j);
                  reg.ReadBinaryData(t, p^, J);
                  for k := 0 to j - 1 do
                  begin
                    Write(f, LowerCase(IntToHex(Byte(p[k]), 2)));
                    if k <> j - 1 then
                    begin
                      Write(f, ',');
                      if (k > 0) and ((k mod 23) = 0) then
                        begin
                        Writeln(f, '\');
                        Write(f, '  ');
                        end;
                    end;
                  end;
                  FreeMem(p, j);
                  Writeln(f);
                end;
              else
                Writeln(f, '""');
            end;
          end;
        finally
          reg.CloseKey;
        end;

      finally
        values.Free;
      end;

      for i := 0 to keys.Count - 1 do
        ProcessBranch(root + '\' + keys[i]);
    finally
      keys.Free;
    end;
  end;

begin
  if RegRoot[Length(Regroot)] = '\' then
    SetLength(regroot, Length(Regroot) - 1);
  Assignfile(f, FileName);
  Rewrite(f);
  if ioResult <> 0 then Exit;
  Writeln(f, 'REGEDIT4');

  reg := TRegistry.Create;
  try
    reg.Rootkey := Rootsection;
    ProcessBranch(Regroot);
  finally
    reg.Free;
    Close(f);
  end;
end;

procedure ExportRegistryBranchToStringList(rootsection: HKEY; Regroot: string;
  SL: TStringList);
var
  reg: TRegistry;
  p: PChar;

  procedure ProcessBranch(root: string);
  var
    values, keys: TStringList;
    i, j, k: longint;
    s, t: string;
  begin
    SL.Add('');
    case rootsection of
      HKEY_CLASSES_ROOT: s   := 'HKEY_CLASSES_ROOT';
      HKEY_CURRENT_USER: s   := 'HKEY_CURRENT_USER';
      HKEY_LOCAL_MACHINE: s  := 'HKEY_LOCAL_MACHINE';
      HKEY_USERS: s          := 'HKEY_USERS';
      HKEY_PERFORMANCE_DATA: s := 'HKEY_PERFORMANCE_DATA';
      HKEY_CURRENT_CONFIG: s := 'HKEY_CURRENT_CONFIG';
      HKEY_DYN_DATA: s       := 'HKEY_DYN_DATA';
    end;
    SL.Add('[' + s + '\' + root + ']');

    reg.OpenKey(root, False);
    try
      values := TStringList.Create;
      try
        keys := TStringList.Create;
        try
          reg.GetValuenames(values);
          reg.GetKeynames(keys);

          for i := 0 to values.Count - 1 do
          begin
            s := values[i];
            t := s;
            if s = '' then s := '@' + '='
            else
            s := '"' + AddBackSlashBeforeQuote(Dblbackslash(s)) + '"' + '=';
            SL.Text := SL.Text + s;

            case reg.Getdatatype(t) of

              rdString:
                SL[SL.Count-1] := SL[SL.Count-1] + '"' + AddBackSlashBeforeQuote(Dblbackslash(reg.ReadString(t))) + '"';

              rdExpandString:
                begin
                  s := Reg.ReadString(t);
                  j := Reg.GetDataSize(t);
                  SL[SL.Count-1] := SL[SL.Count-1] + ('hex(2):');
                  for k := 1 to j do
                  begin
                    SL[SL.Count-1] := SL[SL.Count-1] + LowerCase((IntToHex(Ord(s[k]), 2)));
                    if k <> j then
                    begin
                      SL[SL.Count-1] := SL[SL.Count-1] + (',');
                      if (k > 0) and ((k mod 23) = 0) then
                        begin
                        SL[SL.Count-1] := SL[SL.Count-1] + ('\');
                        SL.Add('');
                        SL[SL.Count-1] := '  ';
                        end;
                    end;
                  end;
                end;

              rdInteger:
                SL[SL.Count-1] := SL[SL.Count-1] + 'dword:' + LowerCase(IntToHex(reg.readinteger(t), 8));

              rdBinary:
                begin
                  SL[SL.Count-1] := SL[SL.Count-1] + ('hex:');
                  j := reg.GetDataSize(t);
                  GetMem(p, j);
                  reg.ReadBinaryData(t, p^, J);
                  for k := 0 to j - 1 do
                  begin
                    SL[SL.Count-1] := SL[SL.Count-1] + LowerCase(IntToHex(Byte(p[k]), 2));
                    if k <> j - 1 then
                    begin
                      SL[SL.Count-1] := SL[SL.Count-1] + (',');
                      if (k > 0) and ((k mod 23) = 0) then
                        begin
                        SL[SL.Count-1] := SL[SL.Count-1] + ('\');
                        SL.Add('');
                        SL[SL.Count-1] := '  ';
                        end;
                    end;
                  end;
                  FreeMem(p, j);
                end;
              else
                begin
                SL.Text := SL.Text + ('""');
                SL.Add('');
                end;
            end;
          end;
        finally
          reg.CloseKey;
        end;

      finally
        values.Free;
      end;

      for i := 0 to keys.Count - 1 do
        ProcessBranch(root + '\' + keys[i]);
    finally
      keys.Free;
    end;
  end;

begin
  if RegRoot[Length(Regroot)] = '\' then
    SetLength(regroot, Length(Regroot) - 1);

  SL.Add('REGEDIT4');

  reg := TRegistry.Create;
  try
    reg.Rootkey := Rootsection;
    ProcessBranch(Regroot);
  finally
    reg.Free;
  end;
end;

end.